@@ -12,7 +12,7 @@ use std::{
 use brickadia::save;
 
 use dashmap::{mapref::entry::Entry, DashMap};
-use events::Event;
+use events::{BrickInteraction, Event};
 use resources::{GhostBrick, Player, PlayerPaint, Plugin, TemplateBounds};
 use serde_json::{json, Value};
 use thiserror::Error;
@@ -195,13 +195,7 @@ impl Omegga {
                             });
                         }
                         "version" => {
-                            let _ = tx.send(Event::Version(
-                                serde_json::from_value::<Vec<String>>(params.unwrap())
-                                    .unwrap()
-                                    .into_iter()
-                                    .next()
-                                    .unwrap(),
-                            ));
+                            let _ = tx.send(Event::Version(params.unwrap()));
                         }
                         "unauthorized" => {
                             let _ = tx.send(Event::Unauthorized);
@@ -273,6 +267,36 @@ impl Omegga {
                                     .map,
                             ));
                         }
+                        "interact" => {
+                            let _ = tx.send(Event::Interact(
+                                serde_json::from_value::<BrickInteraction>(params.unwrap())
+                                    .unwrap(),
+                            ));
+                        }
+                        e if e.starts_with("event:") => {
+                            let e = &e[6..];
+                            match params {
+                                Some(Value::Array(params)) => {
+                                    let mut params = params.into_iter();
+                                    let player =
+                                        serde_json::from_value::<Player>(params.next().unwrap())
+                                            .unwrap();
+                                    let args = params
+                                        .map(|a| String::from(a.as_str().unwrap()))
+                                        .collect::<Vec<_>>();
+
+                                    let _ = tx.send(Event::Event {
+                                        name: String::from(e),
+                                        player,
+                                        args,
+                                    });
+                                }
+                                _ => continue,
+                            }
+                        }
+                        "autorestart" => {
+                            let _ = tx.send(Event::Autorestart(params.unwrap_or_default()));
+                        }
                         _ => (),
                     },
                 };
@@ -334,6 +358,11 @@ impl Omegga {
         ResponseAwaiter(rx)
     }
 
+    /// Register commands with Omegga. Call when the plugin is initialized.
+    pub fn register_commands(&self, id: rpc::RequestId, commands: &[&str]) {
+        self.write_response(id, Some(json!({ "registeredCommands": commands })), None);
+    }
+
     /// Prints a message to the Omegga console.
     pub fn log(&self, line: impl Into<String>) {
         self.write_notification("log", Some(Value::String(line.into())));
@@ -406,6 +435,13 @@ impl Omegga {
         );
     }
 
+    pub fn middle_print(&self, username: impl Into<String>, line: impl Into<String>) {
+        self.write_notification(
+            "middlePrint",
+            Some(json!({"target": username.into(), "line": line.into()})),
+        );
+    }
+
     /// Gets a list of all players.
     pub async fn get_players(&self) -> Result<Vec<Player>, ResponseError> {
         self.request("getPlayers", None).await.map(|r| match r {
@@ -503,6 +539,16 @@ impl Omegga {
         self.request("loadBricks", Some(json!({"name": name.into(), "quiet": quiet, "offX": offset.0, "offY": offset.1, "offZ": offset.2}))).await.map(|_| ())
     }
 
+    /// Load a save onto a player's clipboard.
+    pub async fn load_bricks_on_player(
+        &self,
+        name: impl Into<String>,
+        player: impl Into<String>,
+        offset: (i32, i32, i32),
+    ) -> Result<(), ResponseError> {
+        self.request("loadBricksOnPlayer", Some(json!({"name": name.into(), "player": player.into(), "offX": offset.0, "offY": offset.1, "offZ": offset.2}))).await.map(|_| ())
+    }
+
     /// Reads a save (from a save file), and returns its data.
     #[cfg(not(feature = "brs"))]
     pub async fn read_save_data(
@@ -549,6 +595,28 @@ impl Omegga {
         self.request("loadSaveData", Some(json!({"data": data, "quiet": quiet, "offX": offset.0, "offY": offset.1, "offZ": offset.2}))).await.map(|_| ())
     }
 
+    /// Loads a save (from a JSON value) onto a player's clipboard.
+    #[cfg(not(feature = "brs"))]
+    pub async fn load_save_data_on_player(
+        &self,
+        data: Value,
+        player: impl Into<String>,
+        offset: (i32, i32, i32),
+    ) -> Result<(), ResponseError> {
+        self.request("loadSaveDataOnPlayer", Some(json!({"data": data, "player": player.into(), "offX": offset.0, "offY": offset.1, "offZ": offset.2}))).await.map(|_| ())
+    }
+
+    /// Loads a save (from brickadia-rs save data) onto a player's clipboard.
+    #[cfg(feature = "brs")]
+    pub async fn load_save_data_on_player(
+        &self,
+        data: save::SaveData,
+        player: impl Into<String>,
+        offset: (i32, i32, i32),
+    ) -> Result<(), ResponseError> {
+        self.request("loadSaveDataOnPlayer", Some(json!({"data": data, "player": player.into(), "offX": offset.0, "offY": offset.1, "offZ": offset.2}))).await.map(|_| ())
+    }
+
     /// Changes the map.
     pub async fn change_map(&self, map: impl Into<String>) -> Result<(), ResponseError> {
         self.request("changeMap", Some(Value::String(map.into())))
